//
// JUMA-TRX2 serial I/O Yaesu CAT interfce simulator
//
// Juha Niinikoski, OH2NLT
//
// Test version 12.01.2007
// VFO lock added 18.01.2007
//

#include "juma-trx2.h"				// board definitions
#include "DataEEPROM.h"				// EEPROM I/O definitions, interface to assembler module
#include "lcd-trx2.h"				// LCD specific definitions

// external references
// external functions

extern void putch(char);						// uart I/O
extern int kbhit(void);
extern unsigned char getch(void);
extern unsigned char getche(void);
extern unsigned long getlong(void);
extern void bin2ascii(unsigned long);
extern void putlong( unsigned long); 

extern void draw_s_meter(int);					// LCD functions
extern void draw_bar(int, int);
extern void lcd_putst(register const char *);
extern void set_chgen(void);
extern void initlcd(void);
extern void clear_lcd(void);
extern void set_cur_lcd(unsigned char);
extern void disp_freq(long);
extern void disp_rit(int);
extern void disp_meter(int, int);

extern void init_adc12(void);
extern int convert_adc12(unsigned int adchs);	// ADC

extern void init_spi1(void);						// SPI, main board 74HC595 I/O latch
extern void exchg_data_spi1(unsigned int);


// external data

extern const char ver[];			// system version

extern int batt;					// Powersupply voltage scaled value, measured only when displayed
extern int service_mode;			// service mode selected flag

extern unsigned int long_push;		// long push timer for buttons

extern long freq_in_use;			// frequency (DDS set value) currently in use
extern long rit;					// rit +/- adder

extern int s_meter;					// scaled S-meter value 0 ... 48
extern int pwr_meter;				// scaled S-meter value 0 ... 48
extern int swr_alarm;				// SWR alarm flag
extern int vfo_lock;				// VFO knob tune lock, 0=unlock

extern int PTT_IN;					// PTT status

// EEPROM structures

#include "trx2_eeprom.h"			// get EEPROM structure definitions

extern union
	{
	struct defval defval;
	unsigned int storage[sizeof(struct defval)/2];
	}eeprom;

// system calibration values

extern union
	{
	struct calval calval;
	unsigned int ee[sizeof(struct calval)/2];
	}cal;

// some constants
// N=(2^32/clkin)*f*2
// clkin = fixed 180000000 Hz
// DDS control word N = ((F in * DDS_MULT) >> 24) * 2
// or N = (F in * DDS_MULT) >> 23

//#define DDS_MULT 400319966 // 2^32/clkin * 2^24
#define DDS_MULT 102481911521LL // 2^32/clkin * 2^32

// module variables
static int old_vfo;					// temporary storage for split command

// Global variables

#define CMD_TIMEOUT 50				// Yaesu spec = 200ms, 100*about 2ms
int cmd_timeout;					// RS232 command timeout, 0 = timeout.

#define CMD 4						// opcode table index
unsigned char yaesu_command[5];		// Yaesu CAT command & response, P1=[0], opcode=[4]
int cmd_idx;						// CAT command table index


// clear CAT packet
void clear_cat(void)
	{
	int x;
	for(x=0; x<5; x++)				// clear CAT structure
		{
		yaesu_command[x] = 0;
		}
	}

// send CAT packet to serial line
void send_cat(void)
	{
	int x;
	for(x=0; x<5; x++)				// send five bytes, [0]/P1 first
		{
		putch(yaesu_command[x]);
		}
	}

// Yaesu FT-897 CAT interface emulator
// Subset of Yaesu commands

void serial_yaesu(void)
	{
	int x, temp;
	unsigned long ultemp;
	long long vltemp;					// very long variable for frequency display & DDS calculations


// handle command timeout
	if(cmd_timeout != 0)				// handle command timeout, later IRQ timer ?
		{
		cmd_timeout--;
		if(cmd_timeout == 0)
			{
			cmd_idx = 0;				// if timeout reset command table index
			clear_cat();				// clear table
			} 
		}

// handle input if any
		if(kbhit())
			{
			cmd_timeout = CMD_TIMEOUT;	// reload timeout timer
			if(cmd_idx < 0)				// keep table index within limits
				cmd_idx = 0;
			if(cmd_idx > 4)
				cmd_idx = 4;
	
			yaesu_command[cmd_idx] = getch();	// get input & put to command structure
			cmd_idx++;					// addjust for next input
			if(cmd_idx < 5)
				return;					// command not ready yet

			cmd_idx = 0;				// set rx pointer for new receice

// encode & process commands
			switch(yaesu_command[CMD])
				{

// Read commands
// Read RX Freq & mode
				case 0x03:
					clear_cat();						// clear CAT structure
// calculate display frequency
					vltemp = (long long)freq_in_use * (long long)cal.calval.ref_osc;	  	// calculate display frequency 
//					ultemp = vltemp >> 32;							// display true dds frequency
					ultemp = vltemp >> 33;							// display LO frequency = DDS freq / 2
// convert to BCD format
					//100_million
					yaesu_command[0] = 0;		// allways 0

					//10_million
					yaesu_command[0] = (yaesu_command[0] << 4) + ultemp/10000000;	
					ultemp-=(ultemp/10000000)*10000000;

					//1_million
					yaesu_command[1] = ultemp/1000000;
					ultemp-=(ultemp/1000000)*1000000;

					//100_thousands
					yaesu_command[1] = (yaesu_command[1] << 4) + ultemp/100000;
					ultemp-=(ultemp/100000)*100000;

					//10_thousands
					yaesu_command[2] = ultemp/10000;
					ultemp-=(ultemp/10000)*10000;

					//1_thousands
					yaesu_command[2] = (yaesu_command[2] << 4) + ultemp/1000;
					ultemp-=(ultemp/1000)*1000;

					//hundreds
					yaesu_command[3] = ultemp/100;
					ultemp-=(ultemp/100)*100;

					//tens
					yaesu_command[3] = (yaesu_command[3] << 4) + ultemp/10;
//					ultemp-=(ultemp/10)*10;

					//ones					// not used

// construct mode bits, TRX2 modes =  0=LSB, 1=USB, 2=CW, 3=Tune
					if(eeprom.defval.mode[eeprom.defval.vfo_sel] < 3)
						yaesu_command[4] = eeprom.defval.mode[eeprom.defval.vfo_sel]; 	// TRX2 mode is the same
					else
						yaesu_command[4] = 0;					// treat Tune as LSB mode
// transmit packet
					send_cat();					// send Yaesu CAT formated frequency packet

					break;

// Read RX Status
				case 0xE7:
					clear_cat();				// clear CAT structure
					temp = s_meter / 3;			// scale s-meter down to 0 ... 15
					if(temp > 15)
						temp = 15;				//saturate
					putch(temp);				// send only one byte
					break;

// Read TX Status
				case 0xF7:
					clear_cat();				// clear CAT structure
					temp = pwr_meter / 3;		// scale pwr-meter down to 0 ... 15
					if(temp > 15)
						temp = 15;				//saturate
					yaesu_command[4] = temp;	// set CAT data

					if(PTT_IN == 0)				// set PTT bit if on
						yaesu_command[4] = yaesu_command[4] & 0x80;

					if(swr_alarm !=0)			// set HI SWR flag
						yaesu_command[4] = yaesu_command[4] & 0x40;

					if(eeprom.defval.vfo_sel > 1) // split 
						yaesu_command[4] = yaesu_command[4] & 0x20;

					putch(yaesu_command[4]);	// send only one byte response
//					send_cat();					// send CAT data
					break;

// Set commands
// Split ON
				case 0x02:
					old_vfo = eeprom.defval.vfo_sel; // save current ststus
					eeprom.defval.vfo_sel = 2;				// set split mode
					putch(0x02);				// echo command, send only one byte
					break;

// Split Off
				case 0x82:
					eeprom.defval.vfo_sel = old_vfo;			// restore selection before split
					putch(0x82);				// echo command, send only one byte
					break;

// Toggle VFO select
				case 0x81:
					if(eeprom.defval.vfo_sel != 0)	// VFO (DDS) settings, 0=A, 1=B
						eeprom.defval.vfo_sel = 0;
					else
						eeprom.defval.vfo_sel = 1;	
					break;

// Set Operating mode, P1 = mode bits
				case 0x07:
					if(yaesu_command[0] < 3)		//valid selections  LSB, USB, CW
						eeprom.defval.mode[eeprom.defval.vfo_sel] = yaesu_command[0];
					break;

// Set Frequency
				case 0x01:
// do BCD to bin conversion for setup data
					ultemp = yaesu_command[0] >> 4;						// 100MHz digit
					ultemp = (ultemp*10) + (yaesu_command[0] & 0x0F);	// 10MHz digit

					ultemp = (ultemp*10) + (yaesu_command[1] >> 4);		// 1MHz digit
					ultemp = (ultemp*10) + (yaesu_command[1]& 0x0F);	// 100kHz digit

					ultemp = (ultemp*10) + (yaesu_command[2] >> 4);		// 10kHz digit
					ultemp = (ultemp*10) + (yaesu_command[2]& 0x0F);	// 1kHz digit

					ultemp = (ultemp*10) + (yaesu_command[3] >> 4);		// 100Hz digit
					ultemp = (ultemp*10) + (yaesu_command[3]& 0x0F);	// 10Hz digit
					ultemp = (ultemp*10);								// 1Hz

					ultemp = ultemp + 5;								// round

					if(ultemp > 40000000)								// check limits
						ultemp = 40000000;


// calculate DDS control word
// DDS control word N = ((F in * DDS_MULT) >> 32) * 2
// or N = (F in * DDS_MULT) >> 31

					vltemp = (long long)ultemp * DDS_MULT;
					eeprom.defval.vfo[eeprom.defval.vfo_sel] = vltemp >> 31;		// set frequency to selected VFO

					clear_cat();									// clear CAT structure
					break;

// PTT ON
				case 0x08:
					PTT_OUT = 1;			// TX on
					putch(0x08);			// echo only one byte
					break;

// PTT OFF
				case 0x88:
					PTT_OUT = 0;			// TX off
					putch(0x88);			// echo only one byte
					break;


// VFO Lock ON
				case 0x00:
					vfo_lock = 1;			// VFO Lock ON
					putch(0x00);			// echo only one byte
					break;

// VFO Lock OFF
				case 0x80:
					vfo_lock = 0;			// VFO Lock OFF
					putch(0x80);			// echo only one byte
					break;


				}	// case
			}		// if
	}
